﻿# 自動サムネイル生成、表示用HTMLファイル生成スクリプト
# 実行方法：添付のbatファイルと一緒に、画像があるフォルダにコピーして
# batファイルを実行してください。そうするとこのスクリプトが実行されます。
# このスクリプトが保存されている場所と同じフォルダ内にある画像全てに対して実行されます。


################################
##### 初期設定部分はじまり #####
################################

#出力するHTMLのファイル名を設定。
# $HTMLoptの値は以下の通り。
#  1:下記変数 $HTML_indexnameに設定した値。 
#  上記以外:index_yymmdd-hhmmss.html サムネイル生成日付時刻を埋め込んだファイル名
#  どちらも既にファイルが存在すると強制的に上書きしますので注意。
$HTMLopt=1 
$HTML_indexname="index.html"

#jpgの品質設定。0～100の整数値で設定。
#実用的な範囲は90～100あたり。
$quality=95

#サムネイルのサイズ（縦横のうち大きな方がこのサイズになります。)
$thumb_size_max=1280

#サムネイル用フォルダ名
##フォルダ名は任意に変更ください。""内がそれです。
##既にある場合、その中にあるファイルがサムネイルのファイル名と一致する場合は強制的に上書きされますので注意。
$thumb_folder_name="thumbs"

#サムネイルのファイル名の末尾に付加する文字列
$thumb_footer="-s"

################################
###### 初期設定部分おわり ######
################################

#このスクリプトの保存場所を取得
$scriptPath = Split-Path $MyInvocation.MyCommand.Path -Resolve
write-host "処理対象フォルダ：" $scriptPath

# アセンブリの読み込み(.NETの機能呼び出し。まあおまじないと思ってください。）
[void][Reflection.Assembly]::LoadWithPartialName("System.Drawing")

#サムネイル用フォルダ作成
#既にある場合は無視されます。
$thumb_folder=join-path $scriptPath $thumb_folder_name
New-Item $thumb_folder -ItemType directory -Force

#保存形式と保存画質の初期設定
$myEncoder=[System.Drawing.Imaging.Encoder]::Quality
$encoderParams=New-Object System.Drawing.Imaging.EncoderParameters(1) 
$encoderParams.Param[0] = New-Object System.Drawing.Imaging.EncoderParameter($myEncoder, $quality)
#画像保存形式jpgの読み込み
$myImageCodecInfo = [System.Drawing.Imaging.ImageCodecInfo]::GetImageEncoders()|where {$_.MimeType -eq 'image/jpeg'}

#ファイル一覧を取得。対象はJPGのみ、このファイルがあるフォルダ内のみでサブフォルダは含まない。
$basejpg=Get-ChildItem -Path $scriptpath -Name *.* -Include *.jpg,*.jpeg,*.jpe
write-host "処理対象数：" ([String]($basejpg.count))

#処理後ファイル名記録用連想配列
$path_befaft=@{}

#各ファイルに対してサムネイル作成(リサイズと保存) 実行
$lp0=0 #進行状況表時用変数
foreach ($jpgfile in $basejpg){
    
    #進行状況表示
    $lp0=$lp0+1
    Write-Progress -Activity ("Thumbnail creating... TOTAL:"+[String]$basejpg.count)  -PercentComplete ($lp0 *100 / $basejpg.count)  -CurrentOperation ([String]$lp0+":"+$jpgfile) -status "作成中..." 

    #元画像のフルパス
    $jpgfullpath=Join-Path $scriptPath $jpgfile
    #サムネイル用のフルパスとファイル名生成(フッタをつける)
    $jpgafter1=Split-Path $jpgfile -Leaf
    $jpgafter2= (Get-ChildItem $jpgfullpath).basename
    $jpgafter3= (Get-ChildItem $jpgfullpath).extension
    $jpgafter4=$jpgafter2+$thumb_footer+$jpgafter3
    $jpgafterfull=Join-Path $thumb_folder $jpgafter4

    # 画像ファイルの読み込み
    $image = New-Object System.Drawing.Bitmap($jpgfullpath)
    #縮小先のオブジェクトを生成
    #縦横の大きい方を初期設定$thumb_size_maxにする、アスペクト比保持
    #$thumb_size_maxより小さいならサイズを変えずそのまま。つまりサムネイル生成の意味は無い(笑)。
    $image_x=$image.Width
    $image_y=$image.Height
    if(($image_x -gt $thumb_size_max )-or ($image_y -gt $thumb_size_max )){ 
        if($image_x -gt $image_y){
            $image_y=[int]($image_y*($thumb_size_max/$image_x))
            $image_x=$thumb_size_max
        }else{
            $image_x=[int]($image_x*($thumb_size_max/$image_y))
            $image_y=$thumb_size_max
        }
    }
    #キャンバスサイズの決定
    $canvas = New-Object System.Drawing.Bitmap($image_x,$image_y)
    # 縮小先へ描画
    $graphics = [System.Drawing.Graphics]::FromImage($canvas)
    #高品質リサイズに設定変更。速度最優先ならHighQualityBicubic→NearestNeighborかDefaultに。
    #いわゆるバーコード/2次元コードはNearestNeighbor推奨。他だとボケて読めない。
    $graphics.InterpolationMode=[System.Drawing.Drawing2D.InterpolationMode]::HighQualityBicubic
    $graphics.DrawImage($image, (New-Object System.Drawing.Rectangle(0, 0, $canvas.Width, $canvas.Height)))
    # 保存(jpg,品質はこのスクリプトの頭で設定)
    $canvas.Save($jpgafterfull, $myImageCodecInfo,$encoderParams)
    # オブジェクトの破棄
    $graphics.Dispose()
    $canvas.Dispose()
    $image.Dispose()
    
    #HTML出力用 サムネイルファイルとの対応を連想配列に保存
    $path_befaft.Add($jpgfile,$jpgafter4)

}
#一個も画像がなかったら何もしない。
#あるときはHTML出力を行う。

if($basejpg.count){
 
    #HTML生成
    ##HTMLファイル名の設定(初期設定を参照)
    if($HTMLopt -eq 1){
        $htmlfilename=join-path $scriptPath $HTML_indexname
    }else{
        $nowdate=(get-date -Format "yyyyMMdd-HHmmss")
        $htmlfilename=join-path $scriptPath ("index_" +$nowdate+".html")
    }
    
    ##ファイルに出力
    ###HTMLヘッダ
    Write-Output "<html><head><title>" | Set-Content -Encoding UTF8 $htmlfilename
    Write-Output  (split-path $scriptPath -Leaf) | Out-File -Append -Encoding utf8 $htmlfilename
    Write-Output "</title></head>"| Out-File -Append -Encoding utf8 $htmlfilename
 
    ###HTML本文
    Write-Output "<body> total:" ($basejpg.count) "<HR>"| Out-File -Append -Encoding utf8 $htmlfilename
    #各ファイルに対し出力
    $lp0=0 #ファイル連番表示用カウンタ初期化
    foreach ($jpgfile in $basejpg){
    $lp0=$lp0+1 #ファイル連番表示用カウンタ+1
        #サムネイルのファイルパス
        $outtemp_thumb=join-path $thumb_folder_name $path_befaft[$jpgfile]
        #元のファイルのファイルサイズ、桁区切りありに整形
        $outtemp_fs_str="{0:N0}" -f ((Get-ItemProperty (join-path $scriptPath $jpgfile)).Length)
        #ヒアドキュメントで一気に出力。@"～"@で囲んだ部分がそれです。
        $outtemp=@"
$lp0 <img src="$outtemp_thumb" height=30% align=left ><a href="$jpgfile">$jpgfile をダウンロード（右クリック/長押しの「リンク先を保存」の使用を推奨」</a><br>filesize:$outtemp_fs_str bytes <br clear=all><hr>
"@
        Write-Output $outtemp | Out-File -Append -Encoding utf8 $htmlfilename
    }

    Write-Output "</body>"| Out-File -Append -Encoding utf8 $htmlfilename
    ###HTMLフッタ
    Write-Output "</html>"| Out-File -Append -Encoding utf8 $htmlfilename
}else{
Write-Warning "処理対象となるJPEG画像はありませんでした。"
}
#対象が保存されているフォルダを開く(必要ならコメント文字 ＃を外してください）
#explorer $scriptPath

